/* -*-c-*-
 *
 * $Id: err.h,v 1.2 2002/02/02 19:16:46 mdw Exp $
 *
 * Error reporting
 *
 * (c) 2001 Mark Wooding
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of Jog: Programming for a jogging machine.
 *
 * Jog is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Jog is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jog; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef ERR_H
#define ERR_H

#ifdef __cplusplus
  extern "C" {
#endif

/*----- Header files ------------------------------------------------------*/

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <stdarg.h>

/*----- Error codes -------------------------------------------------------*/

/* --- Abort codes --- *
 *
 * Only for the most serious errors, pertaining to the log file.
 */

#define ERRABORT_LOGOPEN 1u		/* Log file wouldn't open */
#define ERRABORT_LOGWRITE 2u		/* Couldn't write to log file */

/* --- Error contexts and reason codes --- *
 *
 * Most errors are reported as a group of three numeric codes.  These are, in
 * order:
 *
 *   * the `context' code, which describes where whatever it was went wrong
 *     went wrong;
 *
 *   * the `reason' code (specific to a particular context), which describes
 *     what the program was trying to do at the time; and
 *
 *   * the `error' code (again specific to a particular context, but usually
 *     an @errno@ error code), which explains what the problem actually was.
 */

#define ERR_EXC 1u			/* mLib exception error code */
  /* Reason code is zero; error code is exception number */

#define ERR_RXGLUE 2u			/* REXX interface stuff */
#  define ERRRX_SCRIPTREAD 1u		/* Problem reading script file */
#  define ERRRX_INTERP 2u		/* Error code from interpreter */
#  define ERRRX_INIT 3u			/* Initialization error */

#define ERR_RXERR 3u			/* REXX error */
  /* Reason code is line number; error code is REXX error number */

#define ERR_TXPORT 4u			/* Transport switch stuff */
#  define ERRTX_BADTX 1u		/* Unknown transport name */
#  define ERRTX_CREATE 2u		/* Error creating transport */
#  define ERRTX_WRITE 3u		/* Error writing to transport */
#  define ERRTX_READ 4u			/* Read error from transport */
#  define ERRTX_CONFIG 5u		/* Error in configuration */

#define ERR_AUDIO 5u			/* Audio subsystem */
#  define ERRAU_NOTFOUND 1u		/* Requested sample wasn't found */
#  define ERRAU_OPEN 2u			/* Couldn't read sample file */
#  define ERRAU_INIT 3u			/* Couldn't initialize audio */

/*----- Functions provided ------------------------------------------------*/

/* --- @err_abort@ --- *
 *
 * Arguments:	@int reason@ = abort reason code
 *		@unsigned long err@ = abort error code
 *		@const char *msg@ = error message
 *
 * Returns:	Doesn't.
 *
 * Use:		Reports a fatal error.
 */

extern void err_abortv(int /*reason*/, unsigned long /*err*/,
		       const char */*msg*/, va_list */*ap*/);

extern void err_abort(int /*reason*/, unsigned long /*err*/,
		      const char */*msg*/, ...);

/* --- @err_init@ --- *
 *
 * Arguments:	---
 *
 * Returns:	---
 *
 * Use:		Attempts to initialize the logging system.  It is a
 *		catastrophic failure if logging can't start up.
 */

extern void err_init(void);

/* --- @err_report@ --- *
 *
 * Arguments:	@int ctx@ = context code
 *		@int reason@ = reason code
 *		@unsigned long err@ = system error code
 *		@const char *msg@ = textual message to log
 *
 * Returns:	---
 *
 * Use:		Reports an error.  Doesn't abort anything unless something
 *		really serious happens.
 */

extern void err_reportv(int /*ctx*/, int /*reason*/, unsigned long /*err*/,
			const char */*msg*/, va_list */*ap*/);

extern void err_report(int /*ctx*/, int /*reason*/, unsigned long /*err*/,
		       const char */*msg*/, ...);

/* --- @err_log@ --- *
 *
 * Arguments:	 @const char *msg@ = textual message to log
 *
 * Returns:	---
 *
 * Use:		Logs a message.
 */

extern void err_logv(const char */*msg*/, va_list */*ap*/);

extern void err_log(const char */*msg*/, ...);

/*----- That's all, folks -------------------------------------------------*/

#ifdef __cplusplus
  }
#endif

#endif
