/* -*-c-*-
 *
 * $Id: serial.c,v 1.2 2002/01/30 09:23:58 mdw Exp $
 *
 * Common serial functionality
 *
 * (c) 2001 Mark Wooding
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of Jog: Programming for a jogging machine.
 *
 * Jog is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Jog is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jog; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*----- Header files ------------------------------------------------------*/

#include <ctype.h>
#include <stdlib.h>
#include <string.h>

#include "serial.h"

/*----- Main code ---------------------------------------------------------*/

/* --- @getbaud@ --- *
 *
 * Arguments:	@const char *p@ = pointer to string
 *		@const char **pp@ = where to store final pointer
 *
 * Returns:	Baud rate, or @-1@ on failure.
 *
 * Use:		Parses a baud rate, either as a raw number or as an
 *		expression like `9k6'.
 */

static long getbaud(const char *p, const char **pp)
{
  long x = 0;
  long f = 1, s = 10;
  int ok = 0;

  /* --- Main loop --- */

  for (;;) {
    int ch = (unsigned char)*p;

    switch (ch) {
      case 'k':
      case 'K':
	f = 1000;
	goto factor;
      case 'm':
      case 'M':
	f = 1000000;
	goto factor;
      factor:
	if (ok != 1)
	  return (-1);
	x *= f;
	s = 1;
	ok = 2;
	break;
      default:
	if (!isdigit(ch))
	  goto done;
	if (!ok)
	  ok = 1;
	if (ok == 2) {
	  if (f == 1)
	    return (-1);
	  f /= 10;
	}
	x = x * s + (ch - '0') * f;
	break;
    }
    p++;
  }
done:
  if (!ok)
    return (-1);
  if (pp)
    *pp = p;
  return (x);
}

/* --- @getparity@ --- *
 *
 * Arguments:	@const char *p@ = pointer to string
 *		@const char **pp@ = where to store final pointer
 *
 * Returns:	Parity constant or @-1@ on error.
 *
 * Use:		Parses a parity setting.
 */

static int getparity(const char *p, const char **pp)
{
  const char *q;

  for (q = p; isalpha((unsigned char)*q); q++)
    ;
  if (q == p)
    return (-1);
  if (pp)
    *pp = q;
  if (strncmp(p, "none", q - p) == 0)
    return (PARITY_NONE);
  else if (strncmp(p, "odd", q - p) == 0)
    return (PARITY_ODD);
  else if (strncmp(p, "even", q - p) == 0)
    return (PARITY_EVEN);
  else
    return (-1);
}

/* --- @getflow@ --- *
 *
 * Arguments:	@const char *p@ = pointer to string
 *		@const char **pp@ = where to store final pointer
 *
 * Returns:	Flow control constant or @-1@ on error.
 *
 * Use:		Parses a flow control setting.
 */

static int getflow(const char *p, const char **pp)
{
  const char *q;

  for (q = p; isalpha((unsigned char)*q); q++)
    ;
  if (q == p)
    return (-1);
  if (pp)
    *pp = q;
  if (strncmp(p, "none", q - p) == 0)
    return (FLOW_NONE);
  else if (strncmp(p, "soft", q - p) == 0 ||
	   strncmp(p, "xon-xoff", q - p) == 0 ||
	   strncmp(p, "xon/xoff", q - p) == 0 ||
	   strncmp(p, "xonxoff", q - p) == 0)
    return (FLOW_XONXOFF);
  else if (strncmp(p, "hard", q - p) == 0 ||
	   strncmp(p, "cts-rts", q - p) == 0 ||
	   strncmp(p, "cts/rts", q - p) == 0 ||
	   strncmp(p, "ctsrts", q - p) == 0 ||
	   strncmp(p, "rts/cts", q - p) == 0 ||
	   strncmp(p, "rts-cts", q - p) == 0 ||
	   strncmp(p, "rtscts", q - p) == 0)
    return (FLOW_RTSCTS);
  else
    return (-1);
}

/* --- @serial_parse@ --- *
 *
 * Arguments:	@serial_config *sc@ = pointer to serial config structure
 *		@const char *k, *v@ = keyword and value pair
 *
 * Returns:	Zero if OK, or @-1@ on error.
 *
 * Use:		Parses a serial config string into something more
 *		reasonable.  The config structure should have been
 *		initialized to something sensible (e.g., @SERIAL_INIT@)
 *		already.
 */

int serial_parse(serial_config *sc, const char *k, const char *v)
{
  long x;
  const char *q;
  char *qq;

  if (strcmp(k, "baud") == 0 || strcmp(k, "bps") == 0) {
    if (!v || (x = getbaud(v, &q)) < 0 || *q)
      return (-1);
    sc->baud = x;
  } else if (strcmp(k, "data") == 0 || strcmp(k, "data-bits") == 0 ||
	     strcmp(k, "databits") == 0 || strcmp(k, "data-len") == 0 ||
	     strcmp(k, "datalen") == 0 || strcmp(k, "word") == 0 ||
	     strcmp(k, "word-len") == 0 || strcmp(k, "wordlen") == 0) {
    if (!v || (x = strtol(v, &qq, 10)) == 0 || *qq)
      return (-1);
    sc->wordlen = x;
  } else if (strcmp(k, "parity") == 0) {
    if (!v || (x = getparity(v, &q)) < 0 || *q)
      return (-1);
    sc->parity = x;
  } else if (strcmp(k, "stop") == 0 || strcmp(k, "stop-bits") == 0 ||
	     strcmp(k, "stopbits") == 0) {
    if (!v || (x = strtol(v, &qq, 10)) == 0 || *qq)
      return (-1);
    sc->stopbits = x;
  } else if (strcmp(k, "flow") == 0 || strcmp(k, "flow-control") == 0 ||
	     strcmp(k, "flowcontrol") == 0) {
    if (!v || (x = getflow(v, &q)) < 0 || *q)
      return (-1);
    sc->flow = x;
  } else if (strcmp(k, "format") == 0) {
    if (!v)
      return (-1);
    if ((x = strtol(v, &qq, 10)) == 0)
      return (-1);
    v = qq;
    sc->wordlen = x;
    if (*v == '-')
      v++;
    if ((x = getparity(v, &q)) < 0)
      return (-1);
    sc->parity = x;
    v = q;
    if (*v == '-')
      v++;
    if ((x = strtol(v, &qq, 10)) == 0)
      return (-1);
    v = qq;
    sc->stopbits = x;
    if (*v)
      return (-1);
  } else
    return (-1);

  return (0);
}

/*----- Test rig ----------------------------------------------------------*/

#ifdef TEST_RIG

#include <stdio.h>
#include <string.h>

int main(int argc, char *argv[])
{
  int i;
  static const char *ptab[] = { "none", "odd", "even" };
  static const char *ftab[] = { "none", "xon/xoff", "rts/cts" };

  for (i = 1; i < argc; i++) {
    serial_config sc = SERIAL_INIT;
    char *k, *v;

    for (k = strtok(argv[i], ";"); k; k = strtok(0, ";")) {
      if ((v = strchr(k, '=')) != 0)
	*v++ = 0;
      if (serial_parse(&sc, k, v)) {
	printf("invalid serial configuration `%s=%s'\n", k, v);
	goto next;
      }
    }
    printf("%lu %u-%s-%u %s\n", sc.baud, sc.wordlen,
	   ptab[sc.parity], sc.stopbits, ftab[sc.flow]);
  next:;
  }
  return (0);
}

#endif

/*----- That's all, folks -------------------------------------------------*/
